@@ -1,6 +1,6 @@
 #!/usr/bin/bash
 
-BASE=/home/barbie/projects/cpanstats
+BASE=/home/barbie/projects/cpantesters/generate
 
 date
 mkdir -p $BASE/logs
@@ -9,6 +9,6 @@ cd $BASE
 perl bin/cpanstats \
     --config=data/settings.ini     \
     --log=../db/logs/cpanstats.log \
+    --nonstop                      \
     >>logs/cpanstats.out
-perl bin/readstats.pl -c -m >logs/readstats.out
 
